﻿using Ai.AgentServer.Server.Fuel;
using Newtonsoft.Json;
using System.Text;

namespace Ai.AgentServer.Server
{
    public class HttpServer : IHttpServer
    {
        public IHttpClientFactory _httpClientFactory;
        public ILogger _logger { get; set; }
        public IConfiguration configuration1 { get; set; }
        public HttpServer(IHttpClientFactory httpClientFactory, ILogger<HttpServer> logger, IConfiguration configuration)
        {
            _httpClientFactory = httpClientFactory;
            _logger = logger;
            configuration1 = configuration;
        }


        /// <summary>
        ///  讯飞通讯
        /// </summary>
        /// <param name="url"></param>
        /// <returns></returns>
        public async Task<FuelReponse> SendCommon(FuelSendRequest request)
        {
            if (request == null || string.IsNullOrWhiteSpace(request.Type)) return new FuelReponse(5001, request);
            var client = _httpClientFactory.CreateClient("FuelClient");
            var xunfeiUrl = "http://117.48.149.67:8899";
            //配置文件读取
            if (!string.IsNullOrWhiteSpace(configuration1.GetSection("XunFei").Value))
            {
                xunfeiUrl = configuration1.GetSection("XunFei").Value;
            }


            client.BaseAddress = new Uri(xunfeiUrl);
            var requestJson = JsonConvert.SerializeObject(request.BuildDto());
            _logger.LogInformation($"讯飞request：{DateTime.Now}-{requestJson}");
            var requesStr = new StringContent(requestJson, Encoding.UTF8, "application/json");
            try
            {
                var response = await client.PostAsync("/app-control-center/app/manualDrive", requesStr);
                _logger.LogInformation($"讯飞reponse：{DateTime.Now}-{response.IsSuccessStatusCode}");
                if (response != null && response.IsSuccessStatusCode)
                {
                    Console.WriteLine("xunfei Success");
                    Console.WriteLine(await response.Content.ReadAsStringAsync());
                    var basefuelreponse = JsonConvert.DeserializeObject<BaseFuelReponse>(await response.Content.ReadAsStringAsync());
                    return new FuelReponse(basefuelreponse).SetID(request);
                }

            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                _logger.LogError($"讯飞reponse：{DateTime.Now}-{ex.Message} \r\n {requestJson}");

                return new FuelReponse(5001, request);

            }
            return new FuelReponse(5001, request);
        }

        /// <summary>
        /// 返回假数据
        /// </summary>
        /// <param name="request">请求参数</param>
        /// <returns>假数据</returns>
        public FuelReponse GetFackData(FuelSendRequest request)
        {
            FuelReponse fuelReponse = new FuelReponse(200, request);
            Data data = new Data();
            switch (request.Type)
            {
                case "refuel_password_current":
                    data.OilLiters = "10.00";
                    data.OilAmount = "0.00";
                    data.LicensePlate = "京A32012";
                    break;
                case "refuel_no_password_enter":
                    data.OilLiters = "0.00";
                    data.OilAmount = "10.00";
                    data.LicensePlate = "京A32012";
                    break;
                default:
                    data.OilAmount = "0.00";
                    data.OilLiters = "0.00";
                    data.LicensePlate = null;
                    break;
            }
            fuelReponse.Data = data;
            return fuelReponse;
        }

    }
}
