﻿using MQTTnet.Client;
using MQTTnet.Protocol;
using MQTTnet;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Ai.AgentServer.Server.Mqtt_Ext;
using Microsoft.AspNetCore.SignalR;
using MQTTnet.Server;
using Ai.AgentServer.Server;

namespace Ai.AgentServer
{
    /// <summary>
    /// MQTT 客户端
    /// 
    /// 生成 的对象必须，开启 satrt 如需自动订阅主题需，配置 SubClients
    /// </summary>
    public class MqttClientService
    {
        public static IMqttClient _mqttClient;

        public static string _subClients=string.Empty;

        /// <summary>
        /// 启用
        /// </summary>
        /// <param name="config"></param>
 
        public void MqttClientStart(MqttClientConfig config)
        {
            var optionsBuilder = new MqttClientOptionsBuilder()
                .WithTcpServer(config.Host, config.Port) // 要访问的mqtt服务端的 ip 和 端口号
               // 要访问的mqtt服务端的用户名和密码
                .WithClientId(config.ClientId) // 设置客户端id
                .WithCleanSession()
                .WithTlsOptions(new MqttClientTlsOptions
                {
                    UseTls = false  // 是否使用 tls加密
                });

            var clientOptions = optionsBuilder.Build();
            _mqttClient = new MqttFactory().CreateMqttClient();

            _mqttClient.ConnectedAsync += _mqttClient_ConnectedAsync; // 客户端连接成功事件
            _mqttClient.DisconnectedAsync += _mqttClient_DisconnectedAsync; // 客户端连接关闭事件
            _mqttClient.ApplicationMessageReceivedAsync += _mqttClient_ApplicationMessageReceivedAsync; // 收到消息事件

            //添加配置订阅的ID
            _subClients=config.SubClients;
     
            _mqttClient.ConnectAsync(clientOptions);


        }

        /// <summary>
        /// 客户端连接关闭事件
        /// </summary>
        /// <param name="arg"></param>
        /// <returns></returns>
        private Task _mqttClient_DisconnectedAsync(MqttClientDisconnectedEventArgs arg)
        {
            Console.WriteLine($"客户端已断开与服务端的连接……");

            return Task.CompletedTask;
        }

        /// <summary>
        /// 客户端连接成功事件
        /// </summary>
        /// <param name="arg"></param>
        /// <returns></returns>
        private Task _mqttClient_ConnectedAsync(MqttClientConnectedEventArgs arg)
        {
            Console.WriteLine($"客户端已连接服务端……");

            // 订阅消息主题
            // MqttQualityOfServiceLevel: （QoS）:  0 最多一次，接收者不确认收到消息，并且消息不被发送者存储和重新发送提供与底层 TCP 协议相同的保证。
            // 1: 保证一条消息至少有一次会传递给接收方。发送方存储消息，直到它从接收方收到确认收到消息的数据包。一条消息可以多次发送或传递。
            // 2: 保证每条消息仅由预期的收件人接收一次。级别2是最安全和最慢的服务质量级别，保证由发送方和接收方之间的至少两个请求/响应（四次握手）。

            if ( !string.IsNullOrEmpty(_subClients) )
            {
                foreach (var subClient in _subClients.Split(","))
                {
                    _mqttClient.SubscribeAsync($"/ToDispenser/{subClient}", MqttQualityOfServiceLevel.AtLeastOnce);
                }
            }
            return Task.CompletedTask;
        }

        /// <summary>
        /// 收到消息事件
        /// </summary>
        /// <param name="arg"></param>
        /// <returns></returns>
        private Task _mqttClient_ApplicationMessageReceivedAsync(MqttApplicationMessageReceivedEventArgs arg)
        {
            var message = Encoding.UTF8.GetString(arg.ApplicationMessage.Payload);
            Console.WriteLine($"ApplicationMessageReceivedAsync：客户端ID=【{arg.ClientId}】接收到消息。 Topic主题=【{arg.ApplicationMessage.Topic}】 消息=【{message}】 qos等级=【{arg.ApplicationMessage.QualityOfServiceLevel}】");
            if (arg.ApplicationMessage.Topic.Contains("lock"))
            {
            }

            CommonMessageHandle.Handle(arg.ApplicationMessage.Topic, message);
            return Task.CompletedTask;
        }

        public void Publish(string data)
        {
            var message = new MqttApplicationMessage
            {
                Topic = "topic_02",
                Payload = Encoding.Default.GetBytes(data),
                QualityOfServiceLevel = MqttQualityOfServiceLevel.AtLeastOnce,
                Retain = true  // 服务端是否保留消息。true为保留，如果有新的订阅者连接，就会立马收到该消息。
            };
            _mqttClient.PublishAsync(message);
        }


        public void PublishToTopic(string topic,string data)
        {
            var message = new MqttApplicationMessage
            {
                Topic = topic,
                Payload = Encoding.Default.GetBytes(data),
                QualityOfServiceLevel = MqttQualityOfServiceLevel.AtLeastOnce,
                Retain = true  // 服务端是否保留消息。true为保留，如果有新的订阅者连接，就会立马收到该消息。
            };
            _mqttClient.PublishAsync(message);
        }
    }
}
